<?php
namespace App\Security;

use App\Entity\User;
use App\Repository\UserRepository;
use Psr\Log\LoggerInterface;
use Symfony\Component\Security\Core\Exception\BadCredentialsException;
use Symfony\Component\Security\Http\AccessToken\AccessTokenHandlerInterface;
use Symfony\Component\Security\Http\Authenticator\Passport\Badge\UserBadge;
use DateInterval;
use DateTimeImmutable;
use DateTimeZone;

class AccessTokenHandler implements AccessTokenHandlerInterface
{

    public function __construct(private UserRepository $userRepository, private LoggerInterface $logger)
    {}

    public function getUserBadgeFrom(string $accessToken): UserBadge
    {
        if ($accessToken === FALSE) {
            throw new BadCredentialsException('Missing credentials.');
        }

        $sign_seed = sodium_base642bin($_ENV['AUTH_SEED'], SODIUM_BASE64_VARIANT_ORIGINAL);
        $sign_pair = sodium_crypto_sign_seed_keypair($sign_seed);
        $sign_public = sodium_crypto_sign_publickey($sign_pair);
        $message_signed = sodium_base642bin($accessToken, SODIUM_BASE64_VARIANT_URLSAFE);
        $message = sodium_crypto_sign_open($message_signed, $sign_public);

        if ($message === FALSE) {
            throw new BadCredentialsException('Invalid credentials.');
        }

        $arr = explode('|', $message);

        $ts = new DateTimeImmutable($arr[1], new DateTimeZone("UTC"));
        $now = new DateTimeImmutable("now", new DateTimeZone("UTC"));

        $ts = $ts->add(DateInterval::createFromDateString('5 min'));
        if ($ts < $now) {
            throw new BadCredentialsException('Token has already expired.');
        }

        $auth_name = $arr[0];
        return new UserBadge($auth_name, fn (string $id) => $this->getOrCreateUser($id));
    }

    private function getOrCreateUser(string $authName): User
    {
        $user = $this->userRepository->findOneByAuthName($authName);
        if (NULL === $user) {
            $user = new User($authName);
        }
        return $user;
    }
}

